/**
 *   Copyright 2007, 2008 Jérôme DAVID, Université de Nantes, INRIA, Université Pierre Mendès France
 *   
 *   Implication.java is part of AROMA.
 *
 *   AROMA is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU Lesser General Public License as published by
 *   the Free Software Foundation; either version 2 of the License, or
 *   (at your option) any later version.
 *
 *   AROMA is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU Lesser General Public License for more details.
 *
 *   You should have received a copy of the GNU Lesser General Public License
 *   along with AROMA; if not, write to the Free Software
 *   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 *
 */
package fr.inrialpes.exmo.aroma.alignment;

import java.util.Iterator;
import java.util.Set;

import org.semanticweb.owl.align.AlignmentException;

import fr.inrialpes.exmo.align.impl.rel.SubsumeRelation;
import fr.inrialpes.exmo.align.impl.rel.SubsumedRelation;
import fr.inrialpes.exmo.aroma.ontology.entities.AEntity;

/**
 * @author jerome DAVID
 * 
 */
public class Implication extends MatchingRelation {

    public Implication(AEntity ent1, AEntity ent2, double value,
	    boolean orientationLR) throws AlignmentException {
	super(ent1, ent2, orientationLR ? new SubsumedRelation()
		: new SubsumeRelation(), value);
    }

    public AEntity getSource() {
	if (this.relation instanceof SubsumedRelation)
	    return object1;
	return object2;
    }

    public AEntity getDest() {
	if (this.relation instanceof SubsumedRelation)
	    return object2;
	return object1;
    }

    /**
     * Is the implication generative of the implication imp (given in parameter)
     * -> A is generative of B if A.source >= B.source and A.target <= B.target
     * 
     * @param imp
     * @return true if generative of imp, false otherwise
     */
    public boolean isGenerativeOf(MatchingRelation rel) {
	if (!(rel instanceof Implication)) return false;
	Implication imp = (Implication) rel;
	if (object1.isAncestorOf(imp.getSource())
		&& this.object2.isDescendantOf(imp.getDest()))
	    return true;
	return false;
    }

    /**
     * verifies that this implication has no generative implication and best in
     * the set
     * 
     * @param rels
     *            a set of implications
     * @return true if there no implication in imp is generative of this
     */
    public boolean hasBestAndGenerativeImplication(Set<? extends MatchingRelation> rels) {
	Iterator<? extends MatchingRelation> it = rels.iterator();
	while (it.hasNext()) {
	    MatchingRelation current = it.next();
	    if (current.isGenerativeOf(this) && current.getStrength() >= this.strength) {
		return true;
	    }
	}
	return false;
    }

    /**
     * verifies that this implication has no generative implication in the set
     * 
     * @param imp
     *            a set of implications
     * @return true if there no implication in imp is generative of this
     */
    public boolean hasGenerativeImplication(Set<Implication> imp) {
	Iterator<Implication> it = imp.iterator();
	while (it.hasNext()) {
	    Implication current = it.next();
	    if (current.isGenerativeOf(this)) { // && current.getValue() >=
		// this.value) {
		return true;
	    }
	}
	return false;
    }

    /**
     * Is the implication generated by imp equivalent to
     * imp.isGenerativeOf(this)
     * 
     * @param imp
     * @return
     */
    public boolean isGeneratedBy(Implication imp) {
	return imp.isGenerativeOf(this);
    }

    public String toString() {
	return String.valueOf(strength).substring(0,Math.min(String.valueOf(strength).length() - 1, 4));
    }

    public Implication inverse() throws AlignmentException {
	return new Implication(this.object1, this.object2, Double.NaN,
		(this.relation instanceof SubsumedRelation) ? false : true);
    }
}
